# -*- coding: utf-8 -*-
"""Handle instruments endpoints."""
from .apirequest import APIRequest
from .decorators import dyndoc_insert, endpoint
from .responses.instruments import responses
from abc import abstractmethod


class Instruments(APIRequest):
    """Instruments - abstract class to handle instruments endpoint."""

    ENDPOINT = ""
    METHOD = "GET"

    @abstractmethod
    @dyndoc_insert(responses)
    def __init__(self, instrument):
        """Instantiate a Instrument APIRequest instance.

        Parameters
        ----------
        instrument : string (required)
            the instrument to operate on

        params : dict with query parameters
        """
        endpoint = self.ENDPOINT.format(instrument=instrument)
        super(Instruments, self).__init__(endpoint, method=self.METHOD)


@endpoint("v3/instruments/{instrument}/candles")
class InstrumentsCandles(Instruments):
    """Get candle data for a specified Instrument."""

    @dyndoc_insert(responses)
    def __init__(self, instrument, params=None):
        """Instantiate an InstrumentsCandles request.

        Parameters
        ----------
        instrument : string (required)
            the instrument to fetch candle data for

        params : dict
            optional request query parameters, check developer.oanda.com
            for details


        Params example::

            {_v3_instruments_instrument_candles_params}


        Candle data example::

        >>> import oandapyV20
        >>> import oandapyV20.endpoints.instruments as instruments
        >>> client = oandapyV20.API(access_token=...)
        >>> params = ...
        >>> r = instruments.InstrumentsCandles(instrument="DE30_EUR",
        >>>                                    params=params)
        >>> client.request(r)
        >>> print r.response


        Output::

            {_v3_instruments_instrument_candles_resp}

        """
        super(InstrumentsCandles, self).__init__(instrument)
        self.params = params


@endpoint("v3/instruments/{instrument}/orderBook")
class InstrumentsOrderBook(Instruments):
    """Get orderbook data for a specified Instrument."""

    @dyndoc_insert(responses)
    def __init__(self, instrument, params=None):
        """Instantiate an InstrumentsOrderBook request.

        Parameters
        ----------
        instrument : string (required)
            the instrument to fetch candle data for

        params : dict
            optional request query parameters, check developer.oanda.com
            for details


        Params example::

            {_v3_instruments_instrument_orderbook_params}


        OrderBook data example::

        >>> import oandapyV20
        >>> import oandapyV20.endpoints.instruments as instruments
        >>> client = oandapyV20.API(access_token=...)
        >>> params = ...
        >>> r = instruments.InstrumentsOrderBook(instrument="EUR_USD",
        >>>                                      params=params)
        >>> client.request(r)
        >>> print r.response


        Output::

            {_v3_instruments_instrument_orderbook_resp}

        """
        super(InstrumentsOrderBook, self).__init__(instrument)
        self.params = params


@endpoint("v3/instruments/{instrument}/positionBook")
class InstrumentsPositionBook(Instruments):
    """Get positionbook data for a specified Instrument."""

    @dyndoc_insert(responses)
    def __init__(self, instrument, params=None):
        """Instantiate an InstrumentsPositionBook request.

        Parameters
        ----------
        instrument : string (required)
            the instrument to fetch candle data for

        params : dict
            optional request query parameters, check developer.oanda.com
            for details


        Params example::

            {_v3_instruments_instrument_positionbook_params}


        PositionBook data example::

        >>> import oandapyV20
        >>> import oandapyV20.endpoints.instruments as instruments
        >>> client = oandapyV20.API(access_token=...)
        >>> params = ...
        >>> r = instruments.InstrumentsPositionBook(instrument="EUR_USD",
        >>>                                         params=params)
        >>> client.request(r)
        >>> print r.response


        Output::

            {_v3_instruments_instrument_positionbook_resp}

        """
        super(InstrumentsPositionBook, self).__init__(instrument)
        self.params = params



# -*- coding: utf-8 -*-
"""Order related definitions."""

definitions = {
    "OrderType": {
        "MARKET": "A Market Order",
        "LIMIT": "A Limit Order",
        "STOP": "A Stop Order",
        "MARKET_IF_TOUCHED": "A Market-if-touched Order",
        "TAKE_PROFIT": "A Take Profit Order",
        "STOP_LOSS": "A Stop Loss Order",
        "TRAILING_STOP_LOSS": "A Trailing Stop Loss Order"
    },
    "CancellableOrderType": {
        "LIMIT": "A Limit Order",
        "STOP": "A Stop Order",
        "MARKET_IF_TOUCHED": "A Market-if-touched Order",
        "TAKE_PROFIT": "A Take Profit Order",
        "STOP_LOSS": "A Stop Loss Order",
        "TRAILING_STOP_LOSS": "A Trailing Stop Loss Order",
    },
    "OrderState": {
        "PENDING": "The Order is currently pending execution",
        "FILLED": "The Order has been filled",
        "TRIGGERED": "The Order has been triggered",
        "CANCELLED": "The Order has been cancelled",
    },
    "OrderStateFilter": {
        "PENDING": "The orders that are currently pending execution",
        "FILLED": "The orders that have been filled",
        "TRIGGERED": "The orders that have been triggered",
        "CANCELLED": "The orders that have been cancelled",
        "ALL": "The orders that are in any of the possible states: "
               "PENDING, FILLED, TRIGGERED, CANCELLED",
    },
    "TimeInForce": {
        "GTC": "The Order is “Good unTil Cancelled”",
        "GTD": "The Order is “Good unTil Date” and will be cancelled at "
               "the provided time",
        "GFD": "The Order is “Good for Day” and will be cancelled at "
               "5pm New York time",
        "FOK": "The Order must be immediately “Filled Or Killed”",
        "IOC": "The Order must be “Immediately partially filled Or Killed”",
    },
    "OrderPositionFill": {
        "OPEN_ONLY": "When the Order is filled, only allow Positions to be "
                     "opened or extended.",
        "REDUCE_FIRST": "When the Order is filled, always fully reduce an "
                        "existing Position before opening a new Position.",
        "REDUCE_ONLY": "When the Order is filled, only reduce an existing "
                       "Position.",
        "DEFAULT": "When the Order is filled, use REDUCE_FIRST behaviour "
                   "for non-client hedging Accounts, and OPEN_ONLY behaviour "
                   "for client hedging Accounts."
    },
    "OrderTriggerCondition": {
        "DEFAULT": "Trigger an Order the “natural” way: compare its price to "
                   "the ask for long Orders and bid for short Orders",
        "INVERSE": "Trigger an Order the opposite of the “natural” way: "
                   "compare its price the bid for long Orders and ask for "
                   "short Orders.",
        "BID": "Trigger an Order by comparing its price to the bid regardless"
               " of whether it is long or short.",
        "ASK": "Trigger an Order by comparing its price to the ask regardless"
               " of whether it is long or short.",
        "MID": "Trigger an Order by comparing its price to the midpoint "
               "regardless of whether it is long or short."
    }
}



# -*- coding: utf-8 -*-
import six
from abc import ABCMeta, abstractmethod

from .baserequest import BaseRequest
from oandapyV20.types import PriceValue
import oandapyV20.definitions.orders as OD


@six.add_metaclass(ABCMeta)
class OnFill(BaseRequest):
    """baseclass for onFill requests."""

    @abstractmethod
    def __init__(self,
                 timeInForce=OD.TimeInForce.GTC,
                 gtdTime=None,
                 clientExtensions=None):
        super(OnFill, self).__init__()

        if timeInForce not in [OD.TimeInForce.GTC,
                               OD.TimeInForce.GTD,
                               OD.TimeInForce.GFD]:
            raise ValueError("timeInForce: {} invalid".format(timeInForce))

        self._data.update({"timeInForce": timeInForce})

        # optional, but required if
        if timeInForce == OD.TimeInForce.GTD and not gtdTime:
            raise ValueError("gtdTime: value required when timeInForce is GTD")
        self._data.update({"gtdTime": gtdTime})
        self._data.update({"clientExtensions": clientExtensions})


class TakeProfitDetails(OnFill):
    """Representation of the specification for a TakeProfitOrder.

    It is typically used to specify 'take profit details' for the
    'takeProfitOnFill' parameter of an OrderRequest. This way one
    can create the Take Profit Order as a dependency when an order
    gets filled.

    The other way to create a TakeProfitOrder is to create it afterwards
    on an existing trade. In that case you use TakeProfitOrderRequest on
    the trade.
    """

    def __init__(self,
                 price,
                 timeInForce=OD.TimeInForce.GTC,
                 gtdTime=None,
                 clientExtensions=None):
        """Instantiate TakeProfitDetails.

        Parameters
        ----------

        price : float or string (required)
            the price to trigger take profit order

        timeInForce : TimeInForce (required), default TimeInForce.GTC
            the time in force

        gtdTime : DateTime (optional)
            gtdTime is required in case timeInForce == TimeInForce.GTD

        Example
        -------

        >>> import json
        >>> from oandapyV20 import API
        >>> import oandapyV20.endpoints.orders as orders
        >>> from oandapyV20.contrib.requests import (
        >>>     MarketOrderRequest, TakeProfitDetails)
        >>>
        >>> accountID = "..."
        >>> client = API(access_token=...)
        >>> # at time of writing EUR_USD = 1.0740
        >>> # let us take profit at 1.10, GoodTillCancel (default)
        >>> takeProfitOnFillOrder = TakeProfitDetails(price=1.10)
        >>> print(takeProfitOnFillOrder.data)
        {
            "timeInForce": "GTC",
            "price": "1.10000"
        }
        >>> ordr = MarketOrderRequest(
        >>>     instrument="EUR_USD",
        >>>     units=10000,
        >>>     takeProfitOnFill=takeProfitOnFillOrder.data
        >>> )
        >>> # or as shortcut ...
        >>> #   takeProfitOnFill=TakeProfitDetails(price=1.10).data
        >>> print(json.dumps(ordr.data, indent=4))
        {
            "order": {
                "timeInForce": "FOK",
                "instrument": "EUR_USD",
                "units": "10000",
                "positionFill": "DEFAULT",
                "type": "MARKET",
                "takeProfitOnFill": {
                    "timeInForce": "GTC",
                    "price": "1.10000"
                }
            }
        }
        >>> r = orders.OrderCreate(accountID, data=ordr.data)
        >>> rv = client.request(r)
        >>> ...
        """
        super(TakeProfitDetails, self).__init__(
            timeInForce=timeInForce,
            gtdTime=gtdTime,
            clientExtensions=clientExtensions)
        self._data.update({"price": PriceValue(price).value})


class StopLossDetails(OnFill):
    """Representation of the specification for a StopLossOrder.

    It is typically used to specify 'stop loss details' for the
    'stopLossOnFill' parameter of an OrderRequest. This way one
    can create the Stop Loss Order as a dependency when an order
    gets filled.

    The other way to create a StopLossOrder is to create it afterwards
    on an existing trade. In that case you use StopLossOrderRequest on
    the trade.
    """

    def __init__(self,
                 price,
                 timeInForce=OD.TimeInForce.GTC,
                 gtdTime=None,
                 clientExtensions=None):
        """Instantiate StopLossDetails.

        Parameters
        ----------

        price : float or string (required)
            the price to trigger take profit order

        timeInForce : TimeInForce (required), default TimeInForce.GTC
            the time in force

        gtdTime : DateTime (optional)
            gtdTime is required in case timeInForce == TimeInForce.GTD

        clientExtensions : ClientExtensions (optional)


        Example
        -------

            >>> import json
            >>> from oandapyV20 import API
            >>> import oandapyV20.endpoints.orders as orders
            >>> from oandapyV20.contrib.requests import (
            >>>     MarketOrderRequest, StopLossDetails)
            >>>
            >>> accountID = "..."
            >>> client = API(access_token=...)
            >>> # at time of writing EUR_USD = 1.0740
            >>> # let us take profit at 1.10, GoodTillCancel (default)
            >>> stopLossOnFill = StopLossDetails(price=1.06)
            >>> print(stopLossOnFill)
            {
                "timeInForce": "GTC",
                "price": "1.10000"
            }
            >>> ordr = MarketOrderRequest(
            >>>     instrument="EUR_USD",
            >>>     units=10000,
            >>>     stopLossOnFill=stopLossOnFill.data
            >>> )
            >>> # or as shortcut ...
            >>> #   stopLossOnFill=StopLossDetails(price=1.06).data
            >>> print(json.dumps(ordr.data, indent=4))
            >>> r = orders.OrderCreate(accountID, data=ordr.data)
            >>> rv = client.request(r)
            >>> ...
        """
        super(StopLossDetails, self).__init__(
            timeInForce=timeInForce,
            gtdTime=gtdTime,
            clientExtensions=clientExtensions)
        self._data.update({"price": PriceValue(price).value})


class TrailingStopLossDetails(OnFill):
    """Representation of the specification for a TrailingStopLossOrder.

    It is typically used to specify 'trailing stop loss details' for the
    'trailingStopLossOnFill' parameter of an OrderRequest. This way one
    can create the Trailing Stop Loss Order as a dependency when an order
    gets filled.

    The other way to create a TrailingStopLossOrder is to create it afterwards
    on an existing trade. In that case you use TrailingStopLossOrderRequest on
    the trade.
    """

    def __init__(self,
                 distance,
                 timeInForce=OD.TimeInForce.GTC,
                 gtdTime=None,
                 clientExtensions=None):
        """Instantiate TrailingStopLossDetails.

        Parameters
        ----------

        distance : float or string (required)
            the price to trigger trailing stop loss order

        timeInForce : TimeInForce (required), default TimeInForce.GTC
            the time in force

        gtdTime : DateTime (optional)
            gtdTime is required in case timeInForce == TimeInForce.GTD

        clientExtensions : ClientExtensions (optional)


        Example
        -------

            >>> import json
            >>> from oandapyV20 import API
            >>> import oandapyV20.endpoints.orders as orders
            >>> from oandapyV20.contrib.requests import (
            >>>     MarketOrderRequest, TrailingStopLossDetails)
            >>>
            >>> accountID = "..."
            >>> client = API(access_token=...)
            >>> # at time of writing EUR_USD = 1.0740
            >>> # let us take profit at 1.10, GoodTillCancel (default)
            >>> trailingStopLossOnFill = TrailingStopLossDetails(price=1.06)
            >>> print(trailingStopLossOnFill)
            {
                "timeInForce": "GTC",
                "price": "1.10000"
            }
            >>> ordr = MarketOrderRequest(
            >>>     instrument="EUR_USD",
            >>>     units=10000,
            >>>     trailingStopLossOnFill=trailingStopLossOnFill.data
            >>> )
            >>> # or as shortcut ...
            >>> #   ...OnFill=trailingStopLossDetails(price=1.06).data
            >>> print(json.dumps(ordr.data, indent=4))
            >>> r = orders.OrderCreate(accountID, data=ordr.data)
            >>> rv = client.request(r)
            >>> ...
        """
        super(TrailingStopLossDetails, self).__init__(
            timeInForce=timeInForce,
            gtdTime=gtdTime,
            clientExtensions=clientExtensions)
        self._data.update({"distance": PriceValue(distance).value})



# -*- coding: utf-8 -*-

from .baserequest import BaseRequest
from oandapyV20.types import TradeID, PriceValue
from oandapyV20.definitions.orders import TimeInForce, OrderType


class TakeProfitOrderRequest(BaseRequest):
    """create a TakeProfit OrderRequest.

    TakeProfitOrderRequest is used to build the body for a TakeProfitOrder.
    The body can be used to pass to the OrderCreate endpoint.
    """

    def __init__(self,
                 tradeID,
                 price,
                 clientTradeID=None,
                 timeInForce=TimeInForce.GTC,
                 gtdTime=None,
                 clientExtensions=None):
        """
        Instantiate a TakeProfitOrderRequest.

        Parameters
        ----------

        tradeID : string (required)
            the tradeID of an existing trade

        price: float (required)
            the price indicating the target price to close the order.

        Example
        -------

        >>> import json
        >>> from oandapyV20 import API
        >>> import oandapyV20.endpoints.orders as orders
        >>> from oandapyV20.contrib.requests import TakeProfitOrderRequest
        >>>
        >>> accountID = "..."
        >>> client = API(access_token=...)
        >>> ordr = TakeProfitOrderRequest(tradeID="1234",
        >>>                               price=1.22)
        >>> print(json.dumps(ordr.data, indent=4))
        {
            "order": {
                "timeInForce": "GTC",
                "price": "1.22000",
                "type": "TAKE_PROFIT",
                "tradeID": "1234"
            }
        }
        >>> r = orders.OrderCreate(accountID, data=ordr.data)
        >>> rv = client.request(r)
        >>> ...
        """
        super(TakeProfitOrderRequest, self).__init__()

        # allowed: GTC/GFD/GTD
        if timeInForce not in [TimeInForce.GTC,
                               TimeInForce.GTD,
                               TimeInForce.GFD]:
            raise ValueError("timeInForce: {}".format(timeInForce))

        # by default for a TAKE_PROFIT order
        self._data.update({"type": OrderType.TAKE_PROFIT})
        self._data.update({"timeInForce": timeInForce})

        # required
        self._data.update({"tradeID": TradeID(tradeID).value})
        self._data.update({"price": PriceValue(price).value})

        # optional, but required if
        self._data.update({"gtdTime": gtdTime})
        if timeInForce == TimeInForce.GTD and not gtdTime:
            raise ValueError("gtdTime missing")

        # optional
        self._data.update({"clientExtensions": clientExtensions})

    @property
    def data(self):
        """data property.

        return the JSON order body
        """
        return dict({"order": super(TakeProfitOrderRequest, self).data})



"""Responses.

responses serve both testing purpose aswell as dynamic docstring replacement
"""
responses = {
    "_v3_accounts_accountID_transactions": {
        "url": "v3/accounts/{accountID}/transactions",
        "params": {
            "pageSize": 200
        },
        "response": {
            "count": 2124,
            "from": "2016-06-24T21:03:50.914647476Z",
            "lastTransactionID": "2124",
            "pageSize": 100,
            "to": "2016-10-05T06:54:14.025946546Z",
            "pages": [
                "https://api-fxpractice.oanda.com/v3/accounts/"
                "101-004-1435156-001/transactions/idrange?from=1&to=100",
                "https://api-fxpractice.oanda.com/v3/accounts/"
                "101-004-1435156-001/transactions/idrange?from=101&to=200",
                "https://api-fxpractice.oanda.com/v3/accounts/"
                "101-004-1435156-001/transactions/idrange?from=201&to=300",
                "https://api-fxpractice.oanda.com/v3/accounts/"
                "101-004-1435156-001/transactions/idrange?from=301&to=400",
                "https://api-fxpractice.oanda.com/v3/accounts/"
                "101-004-1435156-001/transactions/idrange?from=401&to=500",
                "https://api-fxpractice.oanda.com/v3/accounts/"
                "101-004-1435156-001/transactions/idrange?from=501&to=600",
                "https://api-fxpractice.oanda.com/v3/accounts/"
                "101-004-1435156-001/transactions/idrange?from=601&to=700",
                "https://api-fxpractice.oanda.com/v3/accounts/"
                "101-004-1435156-001/transactions/idrange?from=701&to=800",
                "https://api-fxpractice.oanda.com/v3/accounts/"
                "101-004-1435156-001/transactions/idrange?from=801&to=900",
                "https://api-fxpractice.oanda.com/v3/accounts/"
                "101-004-1435156-001/transactions/idrange?from=901&to=1000",
                "https://api-fxpractice.oanda.com/v3/accounts/"
                "101-004-1435156-001/transactions/idrange?from=1001&to=1100",
                "https://api-fxpractice.oanda.com/v3/accounts/"
                "101-004-1435156-001/transactions/idrange?from=1101&to=1200",
                "https://api-fxpractice.oanda.com/v3/accounts/"
                "101-004-1435156-001/transactions/idrange?from=1201&to=1300",
                "https://api-fxpractice.oanda.com/v3/accounts/"
                "101-004-1435156-001/transactions/idrange?from=1301&to=1400",
                "https://api-fxpractice.oanda.com/v3/accounts/"
                "101-004-1435156-001/transactions/idrange?from=1401&to=1500",
                "https://api-fxpractice.oanda.com/v3/accounts/"
                "101-004-1435156-001/transactions/idrange?from=1501&to=1600",
                "https://api-fxpractice.oanda.com/v3/accounts/"
                "101-004-1435156-001/transactions/idrange?from=1601&to=1700",
                "https://api-fxpractice.oanda.com/v3/accounts/"
                "101-004-1435156-001/transactions/idrange?from=1701&to=1800",
                "https://api-fxpractice.oanda.com/v3/accounts/"
                "101-004-1435156-001/transactions/idrange?from=1801&to=1900",
                "https://api-fxpractice.oanda.com/v3/accounts/"
                "101-004-1435156-001/transactions/idrange?from=1901&to=2000",
                "https://api-fxpractice.oanda.com/v3/accounts/"
                "101-004-1435156-001/transactions/idrange?from=2001&to=2100",
                "https://api-fxpractice.oanda.com/v3/accounts/"
                "101-004-1435156-001/transactions/idrange?from=2101&to=2124"
            ]
        }
    },
    "_v3_accounts_transaction_details": {
        "url": "v3/accounts/{accountID}/transactions/{transactionID}",
        "response": {
            "transaction": {
                "timeInForce": "GTC",
                "triggerCondition": "TRIGGER_DEFAULT",
                "positionFill": "DEFAULT",
                "stopLossOnFill": {
                    "timeInForce": "GTC",
                    "price": "1.22000"
                },
                "userID": 1435156,
                "id": "2304",
                "batchID": "2304",
                "instrument": "EUR_USD",
                "reason": "CLIENT_ORDER",
                "time": "2016-10-24T21:48:18.593753865Z",
                "units": "-100",
                "type": "LIMIT_ORDER",
                "price": "1.20000",
                "accountID": "101-004-1435156-001"
            },
            "lastTransactionID": "2311"
        }
    },
    "_v3_accounts_transaction_idrange": {
        "url": "v3/accounts/{accountID}/transactions/idrange",
        "params": {
            "from": 2304,
            "to": 2306
        },
        "response": {
            "lastTransactionID": "2311",
            "transactions": [
                {
                    "timeInForce": "GTC",
                    "triggerCondition": "TRIGGER_DEFAULT",
                    "positionFill": "DEFAULT",
                    "stopLossOnFill": {
                      "timeInForce": "GTC",
                      "price": "1.22000"
                    },
                    "userID": 1435156,
                    "id": "2304",
                    "batchID": "2304",
                    "instrument": "EUR_USD",
                    "reason": "CLIENT_ORDER",
                    "time": "2016-10-24T21:48:18.593753865Z",
                    "units": "-100",
                    "type": "LIMIT_ORDER",
                    "price": "1.20000",
                    "accountID": "101-004-1435156-001"
                },
                {
                    "orderID": "2304",
                    "userID": 1435156,
                    "batchID": "2305",
                    "clientExtensionsModify": {
                      "comment": "myComment",
                      "id": "myID"
                    },
                    "time": "2016-10-25T15:56:43.075594239Z",
                    "type": "ORDER_CLIENT_EXTENSIONS_MODIFY",
                    "id": "2305",
                    "accountID": "101-004-1435156-001"
                },
                {
                    "orderID": "2304",
                    "clientOrderID": "myID",
                    "replacedByOrderID": "2307",
                    "userID": 1435156,
                    "batchID": "2306",
                    "reason": "CLIENT_REQUEST_REPLACED",
                    "time": "2016-10-25T19:45:38.558056359Z",
                    "type": "ORDER_CANCEL",
                    "id": "2306",
                    "accountID": "101-004-1435156-001"
                }
            ]
        }
    },
    "_v3_accounts_transaction_sinceid": {
        "url": "v3/accounts/{accountID}/transactions/sinceid",
        "params": {
            "id": 2306
        },
        "response": {
            "lastTransactionID": "2311",
            "transactions": [
                {
                  "timeInForce": "GTC",
                  "triggerCondition": "TRIGGER_DEFAULT",
                  "replacesOrderID": "2304",
                  "positionFill": "DEFAULT",
                  "userID": 1435156,
                  "units": "-500000",
                  "batchID": "2306",
                  "instrument": "EUR_USD",
                  "reason": "REPLACEMENT",
                  "time": "2016-10-25T19:45:38.558056359Z",
                  "price": "1.25000",
                  "clientExtensions": {
                    "comment": "myComment",
                    "id": "myID"
                  },
                  "type": "LIMIT_ORDER",
                  "id": "2307",
                  "accountID": "101-004-1435156-001"
                },
                {
                  "orderID": "2307",
                  "clientOrderID": "myID",
                  "userID": 1435156,
                  "batchID": "2308",
                  "reason": "CLIENT_REQUEST",
                  "time": "2016-10-25T20:53:03.789670387Z",
                  "type": "ORDER_CANCEL",
                  "id": "2308",
                  "accountID": "101-004-1435156-001"
                },
                {
                  "timeInForce": "GTC",
                  "triggerCondition": "TRIGGER_DEFAULT",
                  "positionFill": "DEFAULT",
                  "stopLossOnFill": {
                    "timeInForce": "GTC",
                    "price": "1.22000"
                  },
                  "userID": 1435156,
                  "id": "2309",
                  "batchID": "2309",
                  "instrument": "EUR_USD",
                  "reason": "CLIENT_ORDER",
                  "time": "2016-10-25T21:07:21.065554321Z",
                  "units": "-100",
                  "type": "LIMIT_ORDER",
                  "price": "1.20000",
                  "accountID": "101-004-1435156-001"
                },
                {
                  "userID": 1435156,
                  "marginRate": "0.01",
                  "batchID": "2310",
                  "time": "2016-10-26T13:28:00.507651360Z",
                  "type": "CLIENT_CONFIGURE",
                  "id": "2310",
                  "accountID": "101-004-1435156-001"
                },
                {
                  "userID": 1435156,
                  "marginRate": "0.01",
                  "batchID": "2311",
                  "time": "2016-10-26T13:28:13.597103123Z",
                  "type": "CLIENT_CONFIGURE",
                  "id": "2311",
                  "accountID": "101-004-1435156-001"
                }
            ]
        }
    },
    "_v3_accounts_transactions_stream": {
        "url": "v3/accounts/{accountID}/transactions/stream",
        "response": [
            {
                "type": "HEARTBEAT",
                "lastTransactionID": "2311",
                "time": "2016-10-28T11:56:12.002855862Z"
            },
            {
                "type": "HEARTBEAT",
                "lastTransactionID": "2311",
                "time": "2016-10-28T11:56:17.059535527Z"
            },
            {
                "type": "HEARTBEAT",
                "lastTransactionID": "2311",
                "time": "2016-10-28T11:56:22.142256403Z"
            },
            {
                "type": "HEARTBEAT",
                "lastTransactionID": "2311",
                "time": "2016-10-28T11:56:27.238853774Z"
            },
            {
                "type": "HEARTBEAT",
                "lastTransactionID": "2311",
                "time": "2016-10-28T11:56:32.289316796Z"
            }
        ]
    }
}



# -*- coding: utf-8 -*-
"""Handle pricing endpoints."""
from .apirequest import APIRequest
from ..exceptions import StreamTerminated
from .decorators import dyndoc_insert, endpoint
from .responses.pricing import responses
from types import GeneratorType
from abc import abstractmethod


class Pricing(APIRequest):
    """Pricing - class to handle pricing endpoint."""

    ENDPOINT = ""
    METHOD = "GET"

    @abstractmethod
    def __init__(self, accountID):
        """Instantiate a Pricing APIRequest instance.

        Parameters
        ----------
        accountID : string (required)
            the accountID of the account.

        """
        endpoint = self.ENDPOINT.format(accountID=accountID)
        super(Pricing, self).__init__(endpoint, method=self.METHOD)


@endpoint("v3/accounts/{accountID}/pricing")
class PricingInfo(Pricing):
    """Pricing.

    Get pricing information for a specified list of Instruments within
    an account.
    """

    @dyndoc_insert(responses)
    def __init__(self, accountID, params=None):
        """Instantiate a PricingStream APIRequest instance.

        Parameters
        ----------
        accountID : string (required)
            the accountID of the account.

        params : dict (required)
            parameters for the request, check developer.oanda.com for details.

        Example
        -------

        >>> import oandapyV20
        >>> from oandapyV20 import API
        >>> import oandapyV20.endpoints.pricing as pricing
        >>> accountID = "..."
        >>> api = API(access_token="...")
        >>> params = {_v3_accounts_accountID_pricing_params}
        >>> r = pricing.PricingInfo(accountID=accountID, params=params)
        >>> rv = api.request(r)
        >>> print r.response

        Output::


           {_v3_accounts_accountID_pricing_resp}


        """
        super(PricingInfo, self).__init__(accountID)
        self.params = params


@endpoint("v3/accounts/{accountID}/pricing/stream")
class PricingStream(Pricing):
    """PricingStream.

    Get realtime pricing information for a specified list of Instruments.

    """

    STREAM = True

    @dyndoc_insert(responses)
    def __init__(self, accountID, params=None):
        """Instantiate a PricingStream APIRequest instance.

        Parameters
        ----------
        accountID : string (required)
            the accountID of the account.

        params : dict (required)
            parameters for the request, check developer.oanda.com for details.

        Example
        -------

        >>> import oandapyV20
        >>> from oandapyV20 import API
        >>> import oandapyV20.endpoints.pricing as pricing
        >>> accountID = "..."
        >>> api = API(access_token="...")
        >>> params = {_v3_accounts_accountID_pricing_stream_params}
        >>> r = pricing.PricingStream(accountID=accountID, params=params)
        >>> rv = api.request(r)
        >>> maxrecs = 100
        >>> for ticks in r:
        >>>     print json.dumps(R, indent=4),","
        >>>     if maxrecs == 0:
        >>>         r.terminate("maxrecs records received")

        Output::

           {_v3_accounts_accountID_pricing_stream_ciresp}

        """
        super(PricingStream, self).__init__(accountID)
        self.params = params

    def terminate(self, message=""):
        """terminate the stream.

        Calling this method will stop the generator yielding tickrecords.
        A message can be passed optionally.
        """
        if not isinstance(self.response, GeneratorType):
            raise ValueError("request does not contain a stream response")

        self.response.throw(StreamTerminated(message))



# -*- coding: utf-8 -*-
"""Handle orders and pendingOrders endpoints."""
from .apirequest import APIRequest
from .decorators import dyndoc_insert, endpoint
from .responses.orders import responses
from abc import abstractmethod


class Orders(APIRequest):
    """Orders - abstract base class to handle the orders endpoints."""

    ENDPOINT = ""
    METHOD = "GET"
    EXPECTED_STATUS = 0

    @abstractmethod
    @dyndoc_insert(responses)
    def __init__(self, accountID, orderID=None):
        """Instantiate an Orders request.

        Parameters
        ----------
        accountID : string (required)
            id of the account to perform the request on.

        orderID : string
            id of the order to perform the request for.
        """
        endpoint = self.ENDPOINT.format(accountID=accountID, orderID=orderID)
        super(Orders, self).__init__(endpoint, method=self.METHOD,
                                     expected_status=self.EXPECTED_STATUS)


@endpoint("v3/accounts/{accountID}/orders", "POST", 201)
class OrderCreate(Orders):
    """Create an Order for an Account."""

    HEADERS = {"Content-Type": "application/json"}

    @dyndoc_insert(responses)
    def __init__(self, accountID, data):
        """Instantiate an OrderCreate request.

        Parameters
        ----------
        accountID : string (required)
            id of the account to perform the request on.

        data : JSON (required)
            json orderbody to send


        Orderbody example::

            {_v3_accounts_accountID_orders_create_body}

        >>> import oandapyV20
        >>> import oandapyV20.endpoints.orders as orders
        >>> client = oandapyV20.API(access_token=...)
        >>> r = orders.OrderCreate(accountID, data=data)
        >>> client.request(r)
        >>> print r.response

        ::

            {_v3_accounts_accountID_orders_create_resp}

        """
        super(OrderCreate, self).__init__(accountID)
        self.data = data


@endpoint("v3/accounts/{accountID}/orders")
class OrderList(Orders):
    """Create an Order for an Account."""

    @dyndoc_insert(responses)
    def __init__(self, accountID, params=None):
        """Instantiate an OrderList request.

        Parameters
        ----------
        accountID : string (required)
            id of the account to perform the request on.

        params : dict
            optional request query parameters, check developer.oanda.com
            for details


        Example::

        >>> import oandapyV20
        >>> import oandapyV20.endpoints.orders as orders
        >>> client = oandapyV20.API(access_token=...)
        >>> r = orders.OrderList(accountID)
        >>> client.request(r)
        >>> print r.response


        Output::

            {_v3_accounts_accountID_orders_list_resp}

        """
        super(OrderList, self).__init__(accountID)
        self.params = params


@endpoint("v3/accounts/{accountID}/pendingOrders")
class OrdersPending(Orders):
    """List all pending Orders in an Account."""

    @dyndoc_insert(responses)
    def __init__(self, accountID):
        """Instantiate an OrdersPending request.

        Parameters
        ----------
        accountID : string (required)
            id of the account to perform the request on.


        Example::

        >>> import oandapyV20
        >>> import oandapyV20.endpoints.orders as orders
        >>> client = oandapyV20.API(access_token=...)
        >>> r = orders.OrdersPending(accountID)
        >>> client.request(r)
        >>> print r.response


        Output::

            {_v3_accounts_accountID_orders_pending_resp}

        """
        super(OrdersPending, self).__init__(accountID)


@endpoint("v3/accounts/{accountID}/orders/{orderID}")
class OrderDetails(Orders):
    """Get details for a single Order in an Account."""

    @dyndoc_insert(responses)
    def __init__(self, accountID, orderID):
        """Instantiate an OrderDetails request.

        Parameters
        ----------
        accountID : string (required)
            id of the account to perform the request on.

        orderID : string (required)
            id of the order to perform the request on.


        >>> import oandapyV20
        >>> import oandapyV20.endpoints.orders as orders
        >>> client = oandapyV20.API(access_token=...)
        >>> r = orders.OrderDetails(accountID=..., orderID=...)
        >>> client.request(r)
        >>> print r.response

        Output::

            {_v3_accounts_accountID_order_details_resp}

        """
        super(OrderDetails, self).__init__(accountID, orderID)


@endpoint("v3/accounts/{accountID}/orders/{orderID}", "PUT", 201)
class OrderReplace(Orders):
    """OrderReplace.

    Replace an Order in an Account by simultaneously cancelling it and
    createing a replacement Order.
    """

    HEADERS = {"Content-Type": "application/json"}

    @dyndoc_insert(responses)
    def __init__(self, accountID, orderID, data):
        """Instantiate an OrderReplace request.

        Parameters
        ----------
        accountID : string (required)
            id of the account to perform the request on.

        orderID : string (required)
            id of the order to perform the request on.

        data : JSON (required)
            json orderbody to send


        Orderbody example::

            {_v3_accounts_accountID_order_replace_body}

        >>> import oandapyV20
        >>> import oandapyV20.endpoints.orders as orders
        >>> client = oandapyV20.API(access_token=...)
        >>> data = {_v3_accounts_accountID_order_replace_body}
        >>> r = orders.OrderReplace(accountID=..., orderID=..., data=data)
        >>> client.request(r)
        >>> print r.response

        Output::

            {_v3_accounts_accountID_order_replace_resp}

        """
        super(OrderReplace, self).__init__(accountID, orderID)
        self.data = data


@endpoint("v3/accounts/{accountID}/orders/{orderID}/cancel", "PUT")
class OrderCancel(Orders):
    """Cancel a pending Order in an Account."""

    @dyndoc_insert(responses)
    def __init__(self, accountID, orderID):
        """Instantiate an OrdersCancel request.

        Parameters
        ----------
        accountID : string (required)
            id of the account to perform the request on.

        orderID : string (required)
            id of the account to perform the request on.


        Example::

        >>> import oandapyV20
        >>> import oandapyV20.endpoints.orders as orders
        >>> client = oandapyV20.API(access_token=...)
        >>> r = orders.OrderCancel(accountID= ..., orderID=...)
        >>> client.request(r)
        >>> print r.response


        Output::

            {_v3_accounts_accountID_order_cancel_resp}

        """
        super(OrderCancel, self).__init__(accountID, orderID)


@endpoint("v3/accounts/{accountID}/orders/{orderID}/clientExtensions", "PUT")
class OrderClientExtensions(Orders):
    """Update the Client Extensions for an Order in an Account.

    .. warning::
        Do not set, modify or delete clientExtensions if your account
        is associated with MT4.
    """

    HEADERS = {"Content-Type": "application/json"}

    @dyndoc_insert(responses)
    def __init__(self, accountID, orderID, data):
        """Instantiate an OrderCreate request.

        Parameters
        ----------
        accountID : string (required)
            id of the account to perform the request on.

        orderID : string (required)
            id of the order to perform the request on.

        data : JSON (required)
            json orderbody to send


        Orderbody example::

            {_v3_accounts_accountID_order_clientextensions_body}

        >>> import oandapyV20
        >>> import oandapyV20.endpoints.orders as orders
        >>> client = oandapyV20.API(access_token=...)
        >>> r = orders.OrderClientExtensions(accountID, orderID, data=data)
        >>> client.request(r)
        >>> print r.response

        ::

            {_v3_accounts_accountID_order_clientextensions_resp}

        """
        super(OrderClientExtensions, self).__init__(accountID, orderID)
        self.data = data
